
/*
Copyright 2007 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
var stringService = Components.classes["@mozilla.org/intl/stringbundle;1"];
var stringBundle = stringService.createBundle("chrome://prefswitch/locale/global.properties");
var stringStatusBar = stringBundle.GetStringFromName("status-bar");
*/

const ID_STATUS_BAR = "status-bar";
const ID_POPUPSET = "prefswitch-popupset";
const ID_MENUPOPUP = "prefswitch-menupopup";
const ID_PROFILE_MENUPOPUP = "prefswitch-profile-menupopup";
const ID_PROFILE_SEPARATOR = "prefswitch-profile-separator";
const ID_CSS_POPUP = "prefswitch-cssPopup";
const ID_JS_POPUP = "prefswitch-jsPopup";
const ID_SEPARATOR = "prefswitch-separator";
const CLASS_INITIAL = "prefswitch-initial";
const CLASS_INJECT = "prefswitch-inject";

const INJECT_PATH = PREF_PATH + "/inject";

function Overlay()
{
	this.appcontent = null;
	this.control = null;
	this.statusbar = null;
	this.popupset = null;
	this.profileMenupopup = null;
	this.profileSeparator = null;
	this.cssPopup = null;
	this.jsPopup = null;
	this.separator = null;
	this.observer = null;
	this.jsObject = {};

	this.openDialog = function(dialogName)
	{
		if(dialogName == null || dialogName == "") return;
		var url = null;
		var name = null;
		var features = null;
		switch(dialogName)
		{
			case "profile":
				url = "chrome://prefswitch/content/profiledialog.xul";
				name = "profiledialog";
				features = "modal";
				break;
			case "switch":
				url = "chrome://prefswitch/content/switchdialog.xul";
				name = "switchdialog";
				features = "";
				break;
			default: return;
		}
		window.openDialog(url, name, features, this.control);
	};

	this.handleEvent = function(event)
	{
		switch(event.type)
		{
			case "load":
				this.control = new Control(this);
				this.appcontent = document.getElementById("appcontent");
				this.appcontent.addEventListener("DOMContentLoaded", this, false);
				this.statusbar = document.getElementById(ID_STATUS_BAR);
				this.popupset = document.getElementById(ID_POPUPSET);
				this.profileMenupopup = document.getElementById(ID_PROFILE_MENUPOPUP);
				this.profileSeparator = document.getElementById(ID_PROFILE_SEPARATOR);
				this.cssPopup = document.getElementById(ID_CSS_POPUP);
				this.jsPopup = document.getElementById(ID_JS_POPUP);
				this.separator = document.getElementById(ID_SEPARATOR);
				this.updateProfiles();
				this.updateAll();
				this.observer = new Observer(this.control, UPDATE_ALL);
				this.observer.register();
				break;
/*	TEST IF THERE ARE ANY PROBLEMS WITH THE DOMCONTENTLOADED EVENT.
			case "unload":
				this.appcontent.removeEventListener("DOMContentLoaded", this.updateContentLoaded, false);
				var statusbarpanel = this.statusbar.firstChild;
				while(statusbarpanel)
				{
					if(statusbarpanel.getAttribute("class") == CLASS_INITIAL) statusbarpanel.o.unregister();
					statusbarpanel = statusbarpanel.nextSibling;
				}
				break;
*/
			case "DOMContentLoaded":
				for(var key in this.jsObject)
				{
					if(this.jsObject[key] == null) continue; // HOW TO REMOVE AN CHILD? (see this.setJavascript())
					this.injectJavaScript(this.jsObject[key], event.target.defaultView);
				}
		}
	};

	this.updateAll = function()
	{
		this.removeStatusbarpanels();
		this.removePopups(this.popupset);
		this.removeMenuitems(this.cssPopup);
		this.removeMenuitems(this.jsPopup);

		this.addStatusbarpanels();
		this.addMenuitems();
	};

	this.removeStatusbarpanels = function()
	{
		var element = this.separator.previousSibling;
		while(element.getAttribute("class") == CLASS_INITIAL)
		{
			var next = element.previousSibling;
			element.ob.unregister();
			this.statusbar.removeChild(element);
			element = next;
		}
	};

	this.removePopups = function(element)
	{
		var popup = element.firstChild;
		while(popup)
		{
			var next = popup.nextSibling;
			if(popup.tagName == "popup") element.removeChild(popup);
//			else break;
			popup = next;
		}
	};

	this.removeMenuitems = function(element)
	{
		var menuitem = element.firstChild;
		while(menuitem)
		{
			var next = menuitem.nextSibling;
			menuitem.ob.unregister();
			element.removeChild(menuitem);
			menuitem = next;
		}
	};

	this.addStatusbarpanels = function()
	{
		var aSwitch = this.control.getSwitches();
		// CREATE STATUSBARPANELS
		while(aSwitch.length > 0)
		{
			var tSwitch = aSwitch.shift();
			var statusbarpanel = this.createSwitchElement(tSwitch, "statusbarpanel", CLASS_INITIAL, tSwitch.getInitial(), "onclick");
			this.statusbar.insertBefore(statusbarpanel, this.separator);
		}
	};

	this.addMenuitems = function()
	{
		// GET FILES
		var directory = this.control.getDirectory(INJECT_PATH);
		var entries = directory.directoryEntries;
		var aFiles = [];
		while(entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if(entry.isFile()) aFiles.push(entry);
		}
		// CREATE SWITCHES
		var aSwitches = [];
		while(aFiles.length > 0)
		{
			var file = aFiles.shift();
			var tSwitch = new Switch(file.leafName, file.leafName, true, file.leafName);
			tSwitch.setInject(true);
			tSwitch.setInjectPath(file.path);
			var disabledState = new State("Extract " + file.leafName,
				"Extract " + file.leafName, "", "", "lightgrey");
			disabledState.addPreference(new Preference("extensions.prefswitch." + file.leafName,
				BOOLEAN,
				false,
				"Extract " + file.leafName));
			tSwitch.addState(disabledState);
			var enabledState = new State("Inject " + file.leafName,
				"Inject " + file.leafName, "", "", "black");
			enabledState.addPreference(new Preference("extensions.prefswitch." + file.leafName,
				BOOLEAN,
				true,
				"Inject " + file.leafName));
			tSwitch.addState(enabledState);
			aSwitches.push(tSwitch);
		}
		// CREATE MENUITEMS
		while(aSwitches.length > 0)
		{
			var tSwitch = aSwitches.shift();
			var menuitem = this.createSwitchElement(tSwitch, "menuitem", "", tSwitch.getName(), "oncommand");
			menuitem.setAttribute("type", "checkbox");
			switch(tSwitch.getInjectPathExtension())
			{
				case "css": this.cssPopup.appendChild(menuitem); break;
				case "js": this.jsPopup.appendChild(menuitem);
			}
		}
	};

	this.createSwitchElement = function(tSwitch, elementType, className, labelName, eventName)
	{
		var element = document.createElement(elementType);
		element.sw = tSwitch;
		element.ob = new Observer(this.control,
			UPDATE_ONE,
			ROOT,
			tSwitch.getID(),
			element);
		element.ob.register();
		element.setAttribute("class", className);
		element.setAttribute("label", labelName);
		if(tSwitch.isHidden()) element.setAttribute("hidden", "true");
		element.setAttribute("context", ID_MENUPOPUP);
		// APPEND CLICK EVENT OR CREATE POPUP MENU
		if(tSwitch.isBooleanSwitch())
			element.setAttribute(eventName, "overlay.onclick(event, this.sw, this.sw.getState(0), true)");
		else
		{
			var popup = document.createElement("popup");
			var id = "popup-" + tSwitch.getID();
			popup.setAttribute("id", id);
			// ADD MENUITEMS
			for(var i=0; i<tSwitch.getStates().length; i++)
			{
				var menuitem = document.createElement("menuitem");
				var tState = tSwitch.getState(i);
				var description = tState.getDescription();
				menuitem.setAttribute("label", description);
				menuitem.sw = tSwitch;
				menuitem.st = tState;
				menuitem.value = tState.getID();
				menuitem.setAttribute("oncommand", "overlay.onclick(event, this.sw, this.st, false)");
				menuitem.setAttribute("type", "checkbox");
				popup.appendChild(menuitem);
			}
			this.popupset.appendChild(popup);
			element.setAttribute("popup", id);
		}
		this.updateOne(element);
		return element;
	};

	this.updateOne = function(element)
	{
		var tSwitch = element.sw;
		if(!tSwitch.isValid()) return;
		var value = this.control.getValue(ROOT + tSwitch.getID(), tSwitch.getState(0).getID(), STRING);
		var tState = tSwitch.getStateByID(value);
		switch(element.tagName)
		{
			case "statusbarpanel":
				var color = tState.getColor();
				if(color != "") element.setAttribute("style", "color:" + color);
				var tooltiptext = tState.getTooltip();
				if(tooltiptext != "") element.setAttribute("tooltiptext", tooltiptext);
				var popup = document.getElementById("popup-" + tSwitch.getID());
				if(popup == null) break;
				var menuitem = popup.firstChild;
				while(menuitem)
				{
					if(menuitem.value == value) menuitem.setAttribute("checked", "true");
					else menuitem.setAttribute("checked", "false");
					menuitem = menuitem.nextSibling;
				}
				break;
			case "menuitem":
//				if(tSwitch.canInject())
//				{
					var path = tSwitch.getInjectPath();
					var inject = tState.getPreference(0).getValue() == "true";
					switch(tSwitch.getInjectPathExtension())
					{
						case "css": this.injectCSS(path, inject); break;
						case "js": this.setJavascript(tSwitch.getID(), path, inject); break;
					}
					element.setAttribute("checked", "" + inject);
//				}
				break;
		}
	};

	this.injectCSS = function(path, value)
	{
/*		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(path);
		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ios.newFileURI(file);*/

		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI("file://" + path, CHARSET, null); // uri, charset, baseuri
		var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);
		if(value) { if(!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
		else { if(sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
	};

	this.setJavascript = function(key, path, value)
	{
		if(value) this.jsObject[key] = path;
		else this.jsObject[key] = null; // HOW TO REMOVE AN CHILD?
	};

	this.injectJavaScript = function(path, defaultView)
	{
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(path);
		var suc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		suc.charset = CHARSET;
		var data = suc.ConvertToUnicode(this.control.getData(file));
		var data = this.control.getData(file);
		var safeView = new XPCNativeWrapper(defaultView);
		var sandbox = Components.utils.Sandbox(safeView);
		sandbox.window = safeView;
		sandbox.document = safeView.document;
		sandbox.XPathResult = Components.interfaces.nsIDOMXPathResult;
		Components.utils.evalInSandbox(data, sandbox);
	};

	this.onclick = function(event, tSwitch, tState, isBooleanSwitch)
	{
		if(event != null && event.button != null && event.button > 0) return;
		if(isBooleanSwitch)
		{
			var value = this.control.getValue(ROOT + tSwitch.getID(), tSwitch.getState(0).getID(), STRING);
			if(value == tSwitch.getState(0).getID()) tState = tSwitch.getState(1);
			var preference = tState.getPreference(0);
			if(preference.getName().substring(0,22) != "extensions.prefswitch.")
				this.control.setValue(preference.getName(), preference.getValue() == "true", preference.getType());
		}
		else
		{
			for(var i=0; i<tState.getPreferences().length; i++)
			{
				var preference = tState.getPreference(i);
				this.control.setValue(preference.getName(), preference.getValue(), preference.getType());
			}
		}
		this.control.setValue(ROOT + tSwitch.getID(), tState.getID(), STRING);
	};

	this.updateProfiles = function()
	{
		while(this.profileMenupopup.firstChild.getAttribute("id") != ID_PROFILE_SEPARATOR) this.profileMenupopup.removeChild(this.profileMenupopup.firstChild);
		var aProfile = this.control.getProfiles();
		while(aProfile.length > 0)
		{
			var menuitem = document.createElement("menuitem");
			var profile = aProfile.shift();
			menuitem.setAttribute("label", profile.getName());
			menuitem.setAttribute("oncommand", "overlay.loadProfile(this)");
			menuitem.p = profile;
			this.profileMenupopup.insertBefore(menuitem, this.profileSeparator);
		}
	};

	this.loadProfile = function(menuitem)
	{
		var profile = menuitem.p;
		var statusbarpanel = this.statusbar.firstChild;
		while(statusbarpanel)
		{
			switch(statusbarpanel.getAttribute("class"))
			{
				case CLASS_INITIAL:
					var tSwitch = statusbarpanel.sw;
					var tState = tSwitch.getStateByID(profile.getFeature(tSwitch.getID()));
					this.onclick(LOAD_PROFILE, tSwitch, tState, false);
			}
			statusbarpanel = statusbarpanel.nextSibling;
		}
		this.setCheckedProfile(menuitem);
	};

	this.setCheckedProfile = function(menuitem)
	{
		var tMenuitem = this.profileMenupopup.firstChild;
		while(tMenuitem.getAttribute("id") != ID_PROFILE_SEPARATOR)
		{
			if(tMenuitem == menuitem) tMenuitem.setAttribute("checked", "true");
			else tMenuitem.setAttribute("checked", "false");
			tMenuitem = tMenuitem.nextSibling;
		}
	};
}

var overlay = new Overlay();
window.addEventListener("load", overlay, false);
//window.addEventListener("unload", overlay, false);
